/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

// tcp_client.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include "nfsnetauth.h"
#include "nfspin.h"
#include <sys/stat.h>

#define PORT 8080
#define BUFFER_SIZE 4
const char *local_path = "/tmp/local_socket";

const char *notpubkeypath = "/opt/nfssec-netauth/conf/pubkey2.pem";
const char *notprikeypath = "/opt/nfssec-netauth/conf/prikey1.pem";
const char *ukey_enable_conf = "/opt/nfssec-netauth/conf/.ukeycert";

int isHasKey(const char *ukeyconfig)
{
    struct stat st;
    if(stat(ukeyconfig, &st) == 0 && S_ISREG(st.st_mode))
    {
        return 1;
    }
    return 0;
}

int main() {
    int sock = 0;
    struct sockaddr_un serv_addr;

    const char *message = "Hello from client";

    // 创建套接字
    if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
        printf("\n Socket creation error \n");
        return -1;
    }

    serv_addr.sun_family = AF_UNIX;
    strcpy(serv_addr.sun_path, local_path);

    // 连接服务器
    if (connect(sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
        printf("\nConnection Failed \n");
        return -1;
    }


    if(isHasKey(ukey_enable_conf) > 0)
    {
        client_init_withdata(sock);
    }else{
        client_init(sock, notpubkeypath, notprikeypath);
    }
    char input[1024] = {0};
    // 发送消息到服务器

    while(1)
    {
        scanf("%s", input);
        channel_send(sock, input, strlen(input), 0);

        // 读取服务器的响应
        char *buffer = NULL;
        int valread = channel_recv(sock, &buffer, 0);
        buffer = (char *)realloc(buffer, valread+1);
        buffer[valread] = '\0';
        printf("size:%d, from Server: %s\n",valread, buffer);

        free(buffer);
    }
    

    //释放网络资源
    channel_release();
    // 关闭套接字
    close(sock);

    return 0;
}

