/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

// tcp_server.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include "nfsnetauth.h"
#include "nfspin.h"
#include <sys/stat.h>

#define PORT 8080
#define BUFFER_SIZE 1024

const char *local_path = "/tmp/local_socket";

const char *notpubkeypath = "/opt/nfssec-netauth/conf/pubkey1.pem";
const char *notprikeypath = "/opt/nfssec-netauth/conf/prikey2.pem";
const char *ukey_enable_conf = "/opt/nfssec-netauth/conf/.ukeycert";

int isHasKey(const char *ukeyconfig)
{
    struct stat st;
    if(stat(ukeyconfig, &st) == 0 && S_ISREG(st.st_mode))
    {
        return 1;
    }
    return 0;
}

int main() {
    int server_fd, new_socket;
    struct sockaddr_un address;
    int opt = 1;
    const char *hello = "Hello from server";

    // 创建套接字文件描述符
    if ((server_fd = socket(AF_UNIX, SOCK_STREAM, 0)) == 0) {
        perror("socket failed");
        exit(EXIT_FAILURE);
    }

    // 设置端口复用
    if (setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR | SO_REUSEPORT, &opt, sizeof(opt))) {
        perror("setsockopt failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    address.sun_family = AF_UNIX;
    strcpy(address.sun_path, local_path);
    unlink(local_path);
    // 绑定套接字到指定端口
    if (bind(server_fd, (struct sockaddr *)&address, sizeof(address)) < 0) {
        perror("bind failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    // 监听端口，最大连接数为3
    if (listen(server_fd, 3) < 0) {
        perror("listen failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    printf("Server is listening on port %d\n", PORT);

    // 接受客户端连接
    if ((new_socket = accept(server_fd, NULL, NULL)) < 0) {
        perror("accept failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    //获取key
    if(isHasKey(ukey_enable_conf) > 0 )
    {
        if(session_init_withdata(new_socket) < 0)
        {
            printf("session init error\n");
        }
    }else{
        session_init(new_socket, notpubkeypath, notprikeypath);
    }

    while(1)
    {
    // 读取客户端发送的数据
        char *buffer = NULL;
        int valread = channel_recv(new_socket, &buffer, 0);
        buffer = (char *)realloc(buffer, valread+1);
        buffer[valread] = '\0';
        printf("size:%d, from Client: %s\n", valread, buffer);

        // 发送回应给客户端
        channel_send(new_socket, buffer, strlen(buffer), 0);
        free(buffer);
    }

    //释放网络资源
    channel_release();
    // 关闭套接字
    close(new_socket);
    close(server_fd);

    return 0;
}

