/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef NFS_TRUSTCHANNEL_H
#define NFS_TRUSTCHANNEL_H
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#ifdef __cplusplus
extern "C" {
#endif

//server
/*
    author: chj
    brief : server端对新建的链接初始化,连接建立后马上调用,用于交换可信密钥
    params: nfd作为socket函数创建的socket描述符
    Date: 2024-09-25
*/
int session_init(const int nfd, const char *pubkeypath, const char *prikeypath);
int session_init_withdata(const int nfd);

//client
/*
    author: chj
    brief : client端初始化,连接建立后马上调用,用于交换可信密钥
    params: nfd作为socket函数创建的socket描述符
    Date: 2024-09-25
*/
int client_init(const int nfd, const char *pubkeypath, const char *prikeypath);
int client_init_withdata(const int nfd);

//comm
/*
    author: chj
    brief : 替换send函数,数据在发送前会进行加密，用于可信传输
    params: nfd作为socket函数创建的socket描述符,inbuf:函数外创建内存，将数据传入，nlen:收到数据大小
            , flags:与recv函数一致
    ret:    sucess:发出数据大小，fail:失败返回-1
    Date: 2024-09-25
*/
ssize_t channel_send(const int nfd, const void *inbuf, size_t nlen, int flags);

/*
    author: chj
    brief : 替换recv函数,加密数据在接收后会解密，用于可信传输
    params: nfd作为socket函数创建的socket描述符, outbuf:函数内创建内存，将数据传出
            , flags:与recv函数一致
    ret:    sucess:传出收到数据大小，fail:失败返回-1
    Date: 2024-09-25
*/
ssize_t channel_recv(const int nfd, char **outbuf, int flags);

/*
    author: chj
    brief : 释放网络资源
    params: 
    ret:    sucess:成功释放资源，fail:失败返回-1
    Date: 2024-09-27
*/
int channel_release();

/*
    author: chj
    brief : 设置是否打印发送和接收的二进制数据
    params: enable:1,打开，0,关闭
    ret:    
    Date: 2024-09-30
*/
void set_audit_enable(const int enable);
#ifdef __cplusplus
}
#endif


#endif
